<?php

namespace Enwikuna\Enwikuna_Helper\Admin;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Admin\Interfaces\Enwikuna_Helper_Admin_Setup_Wizard_Interface;
use Enwikuna\Enwikuna_Helper\Enwikuna_Helper_Encryptor;

/**
 * Class Enwikuna_Helper_Admin_Setup_Wizard
 *
 * @package Enwikuna\Enwikuna_Helper\Admin
 */
class Enwikuna_Helper_Admin_Setup_Wizard extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Admin_Setup_Wizard_Interface {

	/**
	 * Current step key
	 *
	 * @var string
	 */
	private string $current_step_key = '';

	/**
	 * Steps for the setup wizard
	 *
	 * @var array
	 */
	private array $steps = array();

	/**
	 * The suffix for script & style debugging in WordPress
	 *
	 * @var string
	 */
	private string $scripts_suffix;

	/**
	 * The file path to the admin setup wizard css dir
	 *
	 * @var string
	 */
	private string $admin_setup_wizard_css_dir;

	/**
	 * The file path to the admin setup wizard js dir
	 *
	 * @var string
	 */
	private string $admin_setup_wizard_js_dir;

	/**
	 * Enwikuna_Helper_Admin_Setup_Wizard constructor
	 */
	public function __construct() {
		if ( is_admin() ) {
			$this->scripts_suffix             = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
			$this->admin_setup_wizard_css_dir = Constants::get_constant( 'ENWIKUNA_HELPER_ASSETS_URL' ) . 'css' . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'setup-wizard' . DIRECTORY_SEPARATOR;
			$this->admin_setup_wizard_js_dir  = Constants::get_constant( 'ENWIKUNA_HELPER_ASSETS_URL' ) . 'js' . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'setup-wizard' . DIRECTORY_SEPARATOR;

			$this->init_hooks();
		}
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_action( 'admin_menu', array( $this, 'admin_menu_action' ) );
		add_action( 'admin_init', array( $this, 'admin_init_action' ) );
		add_action( 'admin_post_ewkh_admin_setup_wizard', array( $this, 'admin_post_ewkh_admin_setup_wizard_action' ) );

		// Only load styles and scripts if the current page is the setup wizard
		if ( $this->is_setup_wizard() ) {
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts_action' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_styles_action' ) );
		}
	}

	/**
	 * Introduce Enwikuna Helper setup wizard page
	 */
	public function admin_menu_action(): void {
		add_submenu_page( '', esc_html__( 'Setup', 'enwikuna-helper' ), esc_html__( 'Setup', 'enwikuna-helper' ), 'manage_options', 'enwikuna-helper-setup-wizard', null );
	}

	/**
	 * Initialize & show the setup wizard
	 */
	public function admin_init_action(): void {
		$this->steps = $this->get_setup_wizard_steps();

		if ( isset( $_REQUEST['step'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$step = sanitize_key( wp_unslash( $_REQUEST['step'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			if ( array_key_exists( $step, $this->get_steps() ) ) {
				$this->current_step_key = $step;
			} else {
				$this->current_step_key = current( array_keys( $this->get_steps() ) );
			}
		} else {
			$this->current_step_key = current( array_keys( $this->get_steps() ) );
		}

		if ( ! $this->is_setup_wizard() ) {
			return;
		}

		ob_start();
		Enwikuna_Helper_Admin_Templates::get_instance()->get_admin_setup_wizard();
		exit;
	}

	/**
	 * Register admin setup wizard scripts
	 */
	public function admin_enqueue_scripts_action(): void {
		wp_register_script(
			'enwikuna-helper-admin-setup-wizard',
			$this->admin_setup_wizard_js_dir . 'setup-wizard' . $this->scripts_suffix . '.js',
			array(
				'jquery',
				'clipboard',
			),
			Constants::get_constant( 'ENWIKUNA_HELPER_VERSION' )
		);

		wp_enqueue_script( 'enwikuna-helper-admin-setup-wizard' );

		wp_localize_script(
			'enwikuna-helper-admin-setup-wizard',
			'enwikuna_helper_admin_setup_wizard',
			array(
				'clipboard_js' => array(
					'i18n' => array(
						'copy_success' => esc_js( esc_html__( 'Copied!', 'enwikuna-helper' ) ),
						'copy_failed'  => esc_js( esc_html__( 'Copying failed!', 'enwikuna-helper' ) ),
					),
				),
			)
		);
	}

	/**
	 * Register admin setup wizard styles
	 */
	public function admin_enqueue_styles_action(): void {
		wp_register_style(
			'enwikuna-helper-admin-setup-wizard',
			$this->admin_setup_wizard_css_dir . 'setup-wizard' . $this->scripts_suffix . '.css',
			array(
				'wp-admin',
				'dashicons',
				'install',
			),
			Constants::get_constant( 'ENWIKUNA_HELPER_VERSION' )
		);

		wp_enqueue_style( 'enwikuna-helper-admin-setup-wizard' );
	}

	/**
	 * Returns the current setup wizard step key
	 *
	 * @return string
	 */
	public function get_current_step_key(): string {
		return $this->current_step_key;
	}

	/**
	 * Returns the current setup wizard step index
	 *
	 * @return int
	 */
	public function get_current_step_index(): int {
		return array_search( $this->get_current_step_key(), array_keys( $this->steps ), true ) + 1;
	}

	/**
	 * Returns the setup wizard step index
	 *
	 * @param string $step_key
	 *
	 * @return int
	 */
	public function get_step_index( string $step_key ): int {
		return array_search( $step_key, array_keys( $this->steps ), true ) + 1;
	}

	/**
	 * Returns all steps of the setup wizard
	 *
	 * @return array
	 */
	public function get_steps(): array {
		return $this->steps;
	}

	/**
	 * Returns the URL of a step
	 *
	 * @param string $step_key
	 *
	 * @return string
	 */
	public function get_step_url( string $step_key ): string {
		$step = $this->get_step( $step_key );

		if ( ! $step ) {
			return admin_url();
		}

		return add_query_arg(
			array(
				'step' => $step_key,
			),
			admin_url( 'admin.php?page=enwikuna-helper-setup-wizard' )
		);
	}

	/**
	 * Returns a step from the steps array
	 *
	 * @param string $step_key
	 *
	 * @return array|false
	 */
	public function get_step( string $step_key = '' ) {
		if ( empty( $step_key ) ) {
			$step_key = $this->get_current_step_key();
		}

		if ( array_key_exists( $step_key, $this->steps ) ) {
			return $this->steps[ $step_key ];
		}

		return false;
	}

	/**
	 * Returns the next step key in the wizard
	 *
	 * @return int|string
	 */
	public function get_next_step_key() {
		$next_step_key      = $this->get_current_step_key();
		$current_step_index = $this->get_current_step_index();

		if ( $current_step_index < count( $this->steps ) ) {
			$next_step_index = $current_step_index + 1;

			foreach ( $this->steps as $step_key => $step ) {
				$step_index = $this->get_step_index( $step_key );

				if ( $step_index === $next_step_index ) {
					$next_step_key = $step_key;
				}
			}
		}

		return $next_step_key;
	}

	/**
	 * Returns the error messages for the setup wizard
	 *
	 * @param string $step_key
	 *
	 * @return false|mixed
	 */
	public function get_error_message( string $step_key = '' ) {
		if ( isset( $_GET['error'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$step      = $this->get_step( $step_key );
			$error_key = sanitize_key( wp_unslash( $_GET['error'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			if ( ! empty( $step['errors'] ) && isset( $step['errors'][ $error_key ] ) ) {
				return $step['errors'][ $error_key ];
			}
		}

		return false;
	}

	/**
	 * Returns the content of each step by using it's helper function
	 *
	 * @param string $view
	 */
	public function get_step_content( string $view ): void {
		$this->$view();
	}

	/**
	 * Save action of the wizard when pressing the next button
	 */
	public function admin_post_ewkh_admin_setup_wizard_action(): void {
		check_admin_referer( 'enwikuna-helper-admin-setup-wizard', 'enwikuna-helper-admin-setup-wizard-nonce' );

		$current_step = $this->get_step( $this->get_current_step_key() );

		if ( empty( $current_step ) ) {
			/** @noinspection ForgottenDebugOutputInspection */
			wp_die( esc_html__( 'Action failed. Please refresh the page and retry.', 'enwikuna-helper' ) );
		}

		$this->{$current_step['handler']}();
	}

	/**
	 * Returns the setup wizard welcome view
	 */
	public function view_welcome(): void {
		Enwikuna_helper_Admin_Templates::get_instance()->get_admin_setup_wizard_view( 'welcome' );
	}

	/**
	 * Handles the submit button click actions of the setup wizard welcome view
	 */
	public function view_welcome_handler(): void {
		$this->redirect_step();
	}

	/**
	 * Returns the setup wizard encryptor view
	 */
	public function view_encryptor(): void {
		Enwikuna_helper_Admin_Templates::get_instance()->get_admin_setup_wizard_view( 'encryptor' );
	}

	/**
	 * Handles the submit button click actions of the setup wizard encryptor view
	 */
	public function view_encryptor_handler(): void {
		$current_step_url = $this->get_step_url( $this->get_current_step_key() );
		$next_step_url    = $this->get_step_url( $this->get_next_step_key() );
		$is_key_inserted  = Enwikuna_Helper_Encryptor::get_instance()->auto_insert_key();

		if ( ! $is_key_inserted ) {
			$error_key = 'auto_insert';
		}

		if ( ! empty( $error_key ) ) {
			$redirect = add_query_arg(
				array(
					'error' => $error_key,
				),
				$current_step_url
			);
		} else {
			$redirect = $next_step_url;

			Enwikuna_Helper_Admin_Notices::get_instance()->remove_all_admin_notices();
		}

		$this->redirect_step( $redirect );
	}

	/**
	 * Returns the setup wizard first steps view
	 */
	public function view_first_steps(): void {
		Enwikuna_Helper_Admin_Templates::get_instance()->get_admin_setup_wizard_view( 'first-steps' );
	}

	/**
	 * Returns the items for the first steps list
	 *
	 * @return array
	 */
	public function get_first_steps_list(): array {
		return array(
			array(
				'url'  => esc_url( admin_url( 'index.php?page=enwikuna' ) ),
				'icon' => 'dashicons-products',
				'name' => esc_html__( 'Check your Enwikuna products', 'enwikuna-helper' ),
			),
			array(
				'url'  => esc_url( is_multisite() ? network_admin_url( 'update-core.php' ) : admin_url( 'update-core.php' ) ),
				'icon' => 'dashicons-update',
				'name' => esc_html__( 'Check for updates', 'enwikuna-helper' ),
			),
		);
	}

	/**
	 * Checks if the current page is the Enwikuna Helper setup wizard page
	 *
	 * @return bool
	 */
	private function is_setup_wizard(): bool {
		if ( isset( $_GET['page'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$page = ewkh_clean( wp_unslash( $_GET['page'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			if ( 'enwikuna-helper-setup-wizard' === $page ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Returns the setup wizard steps
	 *
	 * @return array
	 */
	private function get_setup_wizard_steps(): array {
		return array(
			'welcome'     => array(
				'name'        => esc_html__( 'Welcome', 'enwikuna-helper' ),
				'view'        => 'view_welcome',
				'handler'     => 'view_welcome_handler',
				'button_next' => esc_html__( 'Lets start!', 'enwikuna-helper' ),
			),
			'encryptor'   => array(
				'name'        => esc_html__( 'Encryptor', 'enwikuna-helper' ),
				'view'        => 'view_encryptor',
				'handler'     => 'view_encryptor_handler',
				'button_next' => ! Enwikuna_Helper_Encryptor::get_instance()->is_configured() && Enwikuna_Helper_Encryptor::get_instance()->supports_auto_insert_delete() ? esc_html__( 'Insert key', 'enwikuna-helper' ) : '',
				'errors'      => array(
					'auto_insert' => esc_html__( 'Sorry, we were not able to automatically insert the encryptor key into the wp-config.php file. Please follow the manual instructions to configure the encryptor.', 'enwikuna-helper' ),
				),
			),
			'first-steps' => array(
				'name'             => esc_html__( 'First steps', 'enwikuna-helper' ),
				'view'             => 'view_first_steps',
				'button_next'      => esc_html__( 'Enwikuna products', 'enwikuna-helper' ),
				'button_next_link' => esc_url( admin_url( 'index.php?page=enwikuna' ) ),
			),
		);
	}

	/**
	 * Redirects to a step inside a handler function
	 *
	 * @param string $redirect
	 */
	private function redirect_step( string $redirect = '' ): void {
		if ( empty( $redirect ) ) {
			$redirect = $this->get_step_url( $this->get_next_step_key() );
		}

		wp_safe_redirect( $redirect );
		exit;
	}
}
