<?php

namespace Enwikuna\Enwikuna_Helper\Admin\Support\Modules;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Enwikuna\Enwikuna_Helper\Admin\Enwikuna_Helper_Admin_Notices;
use Enwikuna\Enwikuna_Helper\Admin\Support\Enwikuna_Helper_Admin_Support_Page;
use Enwikuna\Enwikuna_Helper\Admin\Support\Modules\Interfaces\Enwikuna_Helper_Admin_Support_Debugger_Interface;

/**
 * Class Enwikuna_Helper_Admin_Support_Debugger
 *
 * @package Enwikuna\Enwikuna_Helper\Admin\Support\Modules
 */
class Enwikuna_Helper_Admin_Support_Debugger extends Enwikuna_Helper_Admin_Support_Page implements Enwikuna_Helper_Admin_Support_Debugger_Interface {

	/**
	 * debug.log path
	 *
	 * @var string|bool
	 */
	private $debug_log_path_file = false;

	/**
	 * wp-config.php path and file
	 *
	 * @var string
	 */
	private string $wp_config_path_file = ABSPATH . '/wp-config.php';

	/**
	 * Constants for debugging
	 *
	 * @var array
	 */
	private array $constants = array(
		'WP_DEBUG',
		'WP_DEBUG_LOG',
		'SCRIPT_DEBUG',
		'SAVEQUERIES',
		'WP_DISABLE_FATAL_ERROR_HANDLER',
	);

	/**
	 * Enwikuna_Helper_Admin_Support_Debugger constructor
	 */
	public function __construct() {
		if ( is_admin() ) {
			$this->id    = 'debugger';
			$this->label = esc_html__( 'Debugger', 'enwikuna-helper' );

			if ( in_array( strtolower( (string) Constants::get_constant( 'WP_DEBUG_LOG' ) ), array( 'true', '1' ), true ) ) {
				$log = trailingslashit( Constants::get_constant( 'WP_CONTENT_DIR' ) ) . 'debug.log';
			} elseif ( is_string( WP_DEBUG_LOG ) ) {
				$log = WP_DEBUG_LOG;
			}

			if ( ! empty( $log ) && file_exists( $log ) ) {
				$this->debug_log_path_file = $log;
			}

			parent::__construct();
		}
	}

	/**
	 * Get support
	 *
	 * @return string
	 */
	public function get_support(): string {
		/**
		 * This filter filters the current admin support ID
		 *
		 * @param string $id The support ID
		 *
		 * @wp-hook ewkh_admin_get_support_{id}
		 * @since 2.0.0
		 */
		return apply_filters( 'ewkh_admin_get_support_' . $this->id, $this->id );
	}

	/**
	 * Returns the debug log path and file
	 *
	 * @return array|false
	 */
	public function get_log() {
		if ( ! file_exists( $this->debug_log_path_file ) ) {
			return false;
		}

		$debug_log_file = file( $this->debug_log_path_file );

		if ( ! empty( $debug_log_file ) ) {
			return array_reverse( $debug_log_file );
		}

		return false;
	}

	/**
	 * Enables the debug logging within the wp-config.php file
	 */
	public function enable_debugging(): void {
		check_admin_referer( 'enwikuna-helper-enable-debug-logging', 'enwikuna-helper-enable-debug-logging-nonce' );

		$wp_config_file    = $this->get_wp_config_file();
		$debugging_enabled = false;

		if ( $wp_config_file ) {
			foreach ( $wp_config_file as $line_num => $line_to_check ) {
				foreach ( $this->constants as $constant_key => $constant ) {
					if ( false !== strpos( $line_to_check, $constant ) ) {
						break;
					}

					if ( false !== strpos( $line_to_check, 'stop editing! Happy publishing' ) || false !== strpos( $line_to_check, 'Absolute path to the WordPress directory' ) ) {
						array_splice( $wp_config_file, $line_num, 0, "define( '$constant', true );\r\n" );

						unset( $this->constants[ $constant_key ] );
					}

					$debugging_enabled = true;
				}
			}

			$wp_config_updated = ewkh_write_to_wp_config( $wp_config_file );

			if ( $debugging_enabled && $wp_config_updated ) {
				Enwikuna_Helper_Admin_Notices::get_instance()->success( esc_html__( 'Debug logging was enabled successfully.', 'enwikuna-helper' ) );
			} else {
				Enwikuna_Helper_Admin_Notices::get_instance()->error( esc_html__( 'An error occurred while trying to enable debug logging.', 'enwikuna-helper' ) );
			}
		}

		wp_safe_redirect( admin_url( 'tools.php?page=enwikuna-support&tab=debugger' ) );
		exit;
	}

	/**
	 * Disables the debug logging within the wp-config.php file
	 */
	public function disable_debugging(): void {
		check_admin_referer( 'enwikuna-helper-disable-debug-logging', 'enwikuna-helper-disable-debug-logging-nonce' );

		$wp_config_file     = $this->get_wp_config_file();
		$debugging_disabled = false;

		if ( $wp_config_file ) {
			foreach ( $wp_config_file as $line_num => $line_to_check ) {
				foreach ( $this->constants as $constant ) {
					if ( false !== strpos( $line_to_check, $constant ) ) {
						unset( $wp_config_file[ $line_num ] );
						$debugging_disabled = true;

						break;
					}
				}
			}

			$wp_config_updated = ewkh_write_to_wp_config( $wp_config_file );

			if ( $debugging_disabled && $wp_config_updated ) {
				Enwikuna_Helper_Admin_Notices::get_instance()->success( esc_html__( 'Debug logging was disabled successfully.', 'enwikuna-helper' ) );
			} else {
				Enwikuna_Helper_Admin_Notices::get_instance()->error( esc_html__( 'An error occurred while trying to disable debug logging.', 'enwikuna-helper' ) );
			}
		}

		wp_safe_redirect( admin_url( 'tools.php?page=enwikuna-support&tab=debugger' ) );
		exit;
	}

	/**
	 * Deletes current debug log
	 */
	public function delete_debug_log(): void {
		check_admin_referer( 'enwikuna-helper-delete-debug-log', 'enwikuna-helper-delete-debug-log-nonce' );

		if ( $this->debug_log_path_file ) {
			unlink( $this->debug_log_path_file );
		}

		wp_safe_redirect( admin_url( 'tools.php?page=enwikuna-support&tab=debugger' ) );
		exit;
	}

	/**
	 * Checks if auto insert/enable/disable of debugging is possible or not
	 *
	 * @return bool
	 */
	public function supports_auto_insert_enable_disable(): bool {
		$supports = false;

		if ( file_exists( $this->wp_config_path_file ) && wp_is_writable( $this->wp_config_path_file ) ) {
			$supports = true;
		}

		return $supports;
	}

	/**
	 * Returns the wp-config.php
	 *
	 * @return array|false
	 */
	private function get_wp_config_file() {
		if ( ! $this->supports_auto_insert_enable_disable() ) {
			Enwikuna_Helper_Admin_Notices::get_instance()->error( esc_html__( 'An error occurred while trying to enable / disable debug logging. The wp-config.php file don\'t exists or is not writable.', 'enwikuna-helper' ) );

			return false;
		}

		$wp_config_file = file( $this->wp_config_path_file );

		if ( ! $wp_config_file ) {
			Enwikuna_Helper_Admin_Notices::get_instance()->error( esc_html__( 'An error occurred while trying to enable / disable debug logging. The wp-debug.log file could not be read.', 'enwikuna-helper' ) );

			return false;
		}

		return $wp_config_file;
	}
}
