<?php

namespace Enwikuna\Enwikuna_Helper\Admin\Support\Modules;

defined( 'ABSPATH' ) || exit;

use Enwikuna\Enwikuna_Helper\Admin\Enwikuna_Helper_Admin_Notices;
use Enwikuna\Enwikuna_Helper\Admin\Support\Enwikuna_Helper_Admin_Support_Page;
use Enwikuna\Enwikuna_Helper\Admin\Support\Modules\Interfaces\Enwikuna_Helper_Admin_Support_Service_User_Interface;
use Enwikuna\Enwikuna_Helper\Enwikuna_Helper_Cron_Jobs;
use Enwikuna\Enwikuna_Helper\Enwikuna_Helper_Service_User;

/**
 * Class Enwikuna_Helper_Admin_Support_Service_User
 *
 * @package Enwikuna\Enwikuna_Helper\Admin\Support\Modules
 */
class Enwikuna_Helper_Admin_Support_Service_User extends Enwikuna_Helper_Admin_Support_Page implements Enwikuna_Helper_Admin_Support_Service_User_Interface {

	/**
	 * The email of the service user
	 *
	 * @var string
	 */
	private string $service_user_email = 'info@enwikuna.de';

	/**
	 * The header used during sending of the service email to Enwikuna
	 *
	 * @var string
	 */
	private string $service_email_header = 'Content-Type: text/html; charset=UTF-8';

	/**
	 * Enwikuna_Helper_Admin_Support_Service_User constructor
	 */
	public function __construct() {
		if ( is_admin() ) {
			$this->id    = 'service-user';
			$this->label = esc_html__( 'Service user', 'enwikuna-helper' );

			parent::__construct();
		}
	}

	/**
	 * Get support
	 *
	 * @return string
	 */
	public function get_support(): string {
		/**
		 * This filter filters the current admin support ID
		 *
		 * @param string $id The support ID
		 *
		 * @wp-hook ewkh_admin_get_support_{id}
		 * @since 2.0.0
		 */
		return apply_filters( 'ewkh_admin_get_support_' . $this->id, $this->id );
	}

	/**
	 * Enables a service user
	 */
	public function enable_service_user(): void {
		check_admin_referer( 'enwikuna-helper-enable-service-user', 'enwikuna-helper-enable-service-user-nonce' );

		$service_user_username = 'enwikuna-support-' . time();
		$service_user_password = wp_generate_password();

		$service_user_id = wp_create_user( $service_user_username, $service_user_password, $this->service_user_email );

		if ( ! $service_user_id || is_wp_error( $service_user_id ) ) {
			/** @noinspection ForgottenDebugOutputInspection */
			wp_die( esc_html__( 'Action failed. Please refresh the page and retry.', 'enwikuna-helper' ) );
		}

		$service_user = get_user_by( 'id', $service_user_id );
		$service_user->add_role( 'administrator' );

		Enwikuna_Helper_Service_User::get_instance()->create(
			array(
				'service_user_id' => $service_user_id,
			)
		);

		Enwikuna_Helper_Cron_Jobs::get_instance()->schedule_recurring( strtotime( '+2 days' ), 'two_days', 'enwikuna_helper_disable_service_user' );

		$subject  = esc_html__( 'Service user enabled on', 'enwikuna-helper' ) . ': ' . get_bloginfo( 'name' );
		$message  = esc_html__( 'User', 'enwikuna-helper' ) . ': ' . $service_user_username . '<br>';
		$message .= esc_html__( 'Password', 'enwikuna-helper' ) . ': ' . $service_user_password . '<br>';
		$message .= esc_html__( 'Login', 'enwikuna-helper' ) . ': ' . wp_login_url();

		wp_mail( $this->service_user_email, $subject, $message, $this->service_email_header );

		wp_safe_redirect( admin_url( 'tools.php?page=enwikuna-support&tab=service-user' ) );
		exit;
	}

	/**
	 * Disables a service user
	 *
	 * @param bool $verify_default_nonce
	 */
	public function disable_service_user( bool $verify_default_nonce = false ): void {
		if ( $verify_default_nonce ) {
			check_admin_referer( 'enwikuna-helper-disable-service-user', 'enwikuna-helper-disable-service-user-nonce' );
		}

		$service_user = Enwikuna_Helper_Service_User::get_instance()->find();

		if ( $service_user ) {
			if ( ! function_exists( 'wp_delete_user' ) ) {
				require_once ABSPATH . 'wp-admin/includes/user.php';
			}

			$service_user_deleted = wp_delete_user( $service_user->get_service_user_id() );

			if ( ! $service_user_deleted ) {
				/** @noinspection ForgottenDebugOutputInspection */
				wp_die( esc_html__( 'Action failed. Please refresh the page and retry.', 'enwikuna-helper' ) );
			}

			$service_user_deleted = Enwikuna_Helper_Service_User::get_instance()->delete_by(
				array(
					'service_user_id' => $service_user->get_service_user_id(),
				)
			);

			if ( ! $service_user_deleted ) {
				/** @noinspection ForgottenDebugOutputInspection */
				wp_die( esc_html__( 'Action failed. Please refresh the page and retry.', 'enwikuna-helper' ) );
			}

			Enwikuna_Helper_Admin_Notices::get_instance()->success( esc_html__( 'The service user was disabled.', 'enwikuna-helper' ) );

			Enwikuna_Helper_Cron_Jobs::get_instance()->clear( 'enwikuna_helper_disable_service_user' );

			$subject = esc_html__( 'Service user disabled on', 'enwikuna-helper' ) . ': ' . get_bloginfo( 'name' );
			$message = esc_html__( 'The service user was disable on', 'enwikuna-helper' ) . ': ' . get_bloginfo( 'name' );

			wp_mail( $this->service_user_email, $subject, $message, $this->service_email_header );
		}

		if ( $verify_default_nonce ) {
			wp_safe_redirect( admin_url( 'tools.php?page=enwikuna-support&tab=service-user' ) );
			exit;
		}
	}
}
