<?php

namespace Enwikuna\Enwikuna_Helper;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Admin\Support\Modules\Enwikuna_Helper_Admin_Support_Service_User;
use Enwikuna\Enwikuna_Helper\Interfaces\Enwikuna_Helper_Cron_Jobs_Interface;

/**
 * Class Enwikuna_Helper_Cron_Jobs
 *
 * @package Enwikuna\Enwikuna_Helper
 */
class Enwikuna_Helper_Cron_Jobs extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Cron_Jobs_Interface {

	/**
	 * Enwikuna_Helper_Cron_Jobs constructor
	 */
	public function __construct() {
		$this->init_hooks();
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded_action' ), 15 );
	}

	/**
	 * Function to create all Cron jobs
	 */
	public function create_cron_jobs(): void {
		// Schedule daily license check
		$this->schedule_recurring( time(), 'hourly', 'enwikuna_helper_license_check' );
	}

	/**
	 * Adds the license check cron after products are initialized
	 */
	public function plugins_loaded_action(): void {
		add_action( 'enwikuna_helper_license_check', array( $this, 'enwikuna_helper_license_check_action' ) );
		add_action( 'enwikuna_helper_disable_service_user', array( $this, 'enwikuna_helper_disable_service_user_action' ) );
		add_action( 'enwikuna_helper_update_callback', array( $this, 'enwikuna_helper_update_callback_action' ) );
	}

	/**
	 * Returns all formatted Enwikuna Helper cron events
	 *
	 * @return array
	 */
	public function get_formatted_cron_events(): array {
		$crons                      = _get_cron_array();
		$enwikuna_helper_cron_hooks = $this->get_cron_hooks();
		$formatted_cron_events      = array();

		if ( ! empty( $crons ) ) {
			foreach ( $crons as $next_run => $cron ) {
				foreach ( $cron as $hook => $dings ) {
					if ( in_array( $hook, $enwikuna_helper_cron_hooks, true ) ) {
						foreach ( $dings as $sig => $data ) {
							$formatted_cron_events[ "$hook-$sig-$next_run" ] = (object) array(
								'hook'     => $hook,
								'next_run' => $next_run,
								'sig'      => $sig,
								'args'     => $data['args'],
								'schedule' => $data['schedule'],
								'interval' => $data['interval'] ?? null,
							);
						}
					}
				}
			}

			// Ensure events are always returned in date descending order
			uasort(
				$formatted_cron_events,
				static function ( $a, $b ) {
					if ( $a->next_run === $b->next_run ) {
						return 0;
					}

					return ( $a->next_run > $b->next_run ) ? 1 : - 1;
				}
			);
		}

		return $formatted_cron_events;
	}

	/**
	 * Checks the license if expired via cron
	 */
	public function enwikuna_helper_license_check_action(): void {
		Enwikuna_Helper::get_instance()->update_registered_products_data( true );

		ewkh_delete_option( 'dismiss_products_expiring_notice' );
	}

	/**
	 * Function to disable the Enwikuna Helper service user
	 */
	public function enwikuna_helper_disable_service_user_action(): void {
		Enwikuna_Helper_Admin_Support_Service_User::get_instance()->disable_service_user();
	}

	/**
	 * Runs the needed updates during installation
	 *
	 * @param string $update_callback
	 */
	public function enwikuna_helper_update_callback_action( string $update_callback ): void {
		include_once Constants::get_constant( 'ENWIKUNA_HELPER_ABSPATH' ) . 'includes' . DIRECTORY_SEPARATOR . 'functions' . DIRECTORY_SEPARATOR . 'enwikuna-helper-update-functions.php';

		if ( function_exists( $update_callback ) && is_callable( $update_callback ) ) {
			$update_callback();
		}
	}

	/**
	 * Schedules a single event
	 *
	 * @param int $timestamp
	 * @param string $hook
	 * @param bool $check_scheduled
	 * @param array $args
	 */
	public function schedule_single( int $timestamp, string $hook, bool $check_scheduled = true, array $args = array() ): void {
		if ( $check_scheduled && $this->is_scheduled( $hook, $args ) ) {
			return;
		}

		wp_schedule_single_event( $timestamp, $hook, $args );
	}

	/**
	 * Schedules a recurring event
	 *
	 * @param int $timestamp
	 * @param string $recurrence
	 * @param string $hook
	 * @param bool $check_scheduled
	 * @param array $args
	 */
	public function schedule_recurring( int $timestamp, string $recurrence, string $hook, bool $check_scheduled = true, array $args = array() ): void {
		if ( $check_scheduled && $this->is_scheduled( $hook, $args ) ) {
			return;
		}

		wp_schedule_event( $timestamp, $recurrence, $hook, $args );
	}

	/**
	 * Clears scheduled hook
	 *
	 * @param string $hook
	 *
	 * @return void
	 */
	public function clear( string $hook ): void {
		wp_unschedule_hook( $hook );
	}

	/**
	 * Check if an event is already scheduled
	 *
	 * @param string $hook
	 * @param array $args
	 *
	 * @return bool
	 */
	public function is_scheduled( string $hook, array $args = array() ): bool {
		if ( ! wp_next_scheduled( $hook, $args ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Checks if the plugin is currently updating
	 *
	 * @return bool
	 */
	public function is_updating(): bool {
		$cron_events = $this->get_formatted_cron_events();

		if ( ! empty( $cron_events ) ) {
			foreach ( $cron_events as $cron_event_key => $cron_event ) {
				if ( 0 === strpos( $cron_event_key, 'enwikuna_helper_update_callback' ) ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Returns a list of all cron hooks used in the plugin
	 *
	 * @return array
	 */
	public function get_cron_hooks(): array {
		return array(
			'enwikuna_helper_license_check',
			'enwikuna_helper_disable_service_user',
			'enwikuna_helper_update_callback',
		);
	}
}
