<?php

namespace Enwikuna\Enwikuna_Helper;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Defuse\Crypto\Crypto;
use Defuse\Crypto\Exception\EnvironmentIsBrokenException;
use Defuse\Crypto\Key;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Interfaces\Enwikuna_Helper_Encryptor_Interface;
use Exception;

/**
 * Class Enwikuna_Helper_Encryptor
 *
 * @package Enwikuna\Enwikuna_Helper
 */
class Enwikuna_Helper_Encryptor extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Encryptor_Interface {

	/**
	 * wp-config.php path and file
	 *
	 * @var string
	 */
	private string $wp_config_path_file = ABSPATH . '/wp-config.php';

	/**
	 * Constant for encryptor key
	 *
	 * @var string
	 */
	private string $constant = 'ENWIKUNA_HELPER_ENCRYPTOR_KEY';

	/**
	 * Maybe encrypt data
	 *
	 * @param string|null $value
	 *
	 * @return string|null
	 */
	public function maybe_encrypt( ?string $value ): ?string {
		if ( empty( $value ) ) {
			return $value;
		}

		if ( $this->is_configured() ) {
			try {
				return Crypto::encrypt( $value, $this->get_key() );
			} catch ( EnvironmentIsBrokenException $e ) {
				return $value;
			}
		}

		return $value;
	}

	/**
	 * Maybe decrypt data
	 *
	 * @param string|null $value
	 *
	 * @return string|null
	 */
	public function maybe_decrypt( ?string $value ): ?string {
		if ( empty( $value ) ) {
			return $value;
		}

		if ( $this->is_configured() ) {
			try {
				$decrypted_value = Crypto::decrypt( $value, $this->get_key() );

				if ( ! empty( $decrypted_value ) ) {
					return $decrypted_value;
				}

				return $value;
			} catch ( Exception $e ) {
				return $value;
			}
		}

		return $value;
	}

	/**
	 * Checks if the encryptor is configured
	 *
	 * @return bool
	 */
	public function is_configured(): bool {
		if ( empty( $this->get_key() ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Returns a random encryptor key
	 *
	 * @param bool $for_config
	 *
	 * @return false|string
	 */
	public function get_random_key( bool $for_config = false ) {
		try {
			$random_key = Key::createNewRandomKey();

			if ( $random_key ) {
				$random_key_ascii_safe = $random_key->saveToAsciiSafeString();

				if ( $for_config ) {
					$random_key_ascii_safe = "define( '$this->constant', '$random_key_ascii_safe' );";
				}

				return $random_key_ascii_safe;
			}

			return false;
		} catch ( EnvironmentIsBrokenException $e ) {
			return false;
		}
	}

	/**
	 * Checks if auto insert/delete of encryptor key within wp-config.php is possible or not
	 *
	 * @return bool
	 */
	public function supports_auto_insert_delete(): bool {
		$supports = false;

		if ( file_exists( $this->wp_config_path_file ) && wp_is_writable( $this->wp_config_path_file ) ) {
			$supports = true;
		}

		/**
		 * This filter filters if the encryptor supports auto insert delete of the key
		 *
		 * @param bool $supports If is supported
		 *
		 * @wp-hook ewkh_encryptor_supports_auto_insert_delete
		 * @since 2.0.0
		 */
		return apply_filters( 'ewkh_encryptor_supports_auto_insert_delete', $supports );
	}

	/**
	 * Inserts the encryptor key into the wp-config.php
	 */
	public function auto_insert_key(): bool {
		if ( empty( $this->get_key() ) ) {
			$random_key = $this->get_random_key( true );

			if ( ! $random_key || ! $this->supports_auto_insert_delete() ) {
				return false;
			}

			$wp_config_file = file( $this->wp_config_path_file );

			if ( ! $wp_config_file ) {
				return false;
			}

			foreach ( $wp_config_file as $line_num => $line_to_check ) {
				if ( 0 === strpos( trim( $line_to_check ), '//' ) ) {
					continue;
				}

				if ( false !== strpos( $line_to_check, $this->constant ) ) {
					return true; // Key is already inserted
				}

				if ( false !== strpos( $line_to_check, 'stop editing! Happy publishing' ) || false !== strpos( $line_to_check, 'Absolute path to the WordPress directory' ) ) {
					array_splice( $wp_config_file, $line_num, 0, "$random_key\r\n" );

					return ewkh_write_to_wp_config( $wp_config_file );
				}
			}
		}

		return true;
	}

	/**
	 * Deletes the encryptor key from the wp-config.php
	 *
	 * @return bool
	 */
	public function auto_delete_key(): bool {
		if ( ! $this->supports_auto_insert_delete() ) {
			return false;
		}

		$wp_config_file = file( $this->wp_config_path_file );

		if ( ! $wp_config_file ) {
			return false;
		}

		foreach ( $wp_config_file as $line_num => $line_to_check ) {
			if ( false !== strpos( $line_to_check, $this->constant ) ) {
				unset( $wp_config_file[ $line_num ] );

				return ewkh_write_to_wp_config( $wp_config_file );
			}
		}

		return false;
	}

	/**
	 * Returns the encryptor key from the wp-config.php
	 *
	 * @return Key|false
	 */
	private function get_key() {
		$key = Constants::get_constant( $this->constant );

		if ( empty( $key ) ) {
			return false;
		}

		try {
			return Key::loadFromAsciiSafeString( $key );
		} catch ( Exception $e ) {
			return false;
		}
	}
}
