<?php

namespace Enwikuna\Enwikuna_Helper;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Interfaces\Enwikuna_Helper_Install_Interface;
use wpdb;

/**
 * Class Enwikuna_Helper_Install
 *
 * @package Enwikuna\Enwikuna_Helper
 */
class Enwikuna_Helper_Install extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Install_Interface {

	/**
	 * Support table identifier
	 *
	 * @var string
	 */
	public const SUPPORT_TABLE_NAME = 'enwikuna_helper_support';

	/**
	 * WordPress database
	 *
	 * @var wpdb
	 */
	private wpdb $wpdb;

	/**
	 * Current version of Enwikuna Helper
	 *
	 * @var string
	 */
	private string $current_enwikuna_helper_version;

	/**
	 * DB updates and callbacks that need to be run per version
	 *
	 * @var array
	 */
	private array $version_updates = array(
		'1.2.0' => array(
			'ewkh_update_120_encryptor_setup',
			'ewkh_update_120_re_encrypt_data',
		),
		'2.2.0' => array(
			'ewkh_update_220_set_host_url',
		),
	);

	/**
	 * Enwikuna_Helper_Install constructor
	 */
	public function __construct() {
		global $wpdb; //DB

		$this->wpdb                            = $wpdb;
		$this->current_enwikuna_helper_version = ewkh_get_option( 'version' );

		$this->init_hooks();
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_filter( 'plugin_row_meta', array( $this, 'filter_plugin_row_meta' ), 10, 2 );
		add_action( 'admin_init', array( $this, 'admin_init_action' ) );
	}

	/**
	 * Install Enwikuna Helper
	 */
	public function install(): void {
		if ( ! is_blog_installed() ) {
			return;
		}

		// Check if we are not already running this routine.
		if ( ewkh_str_to_bool( ewkh_get_transient( 'installing' ) ) ) {
			return;
		}

		// If installation not running, run it now
		ewkh_set_transient( 'installing', 'yes', MINUTE_IN_SECONDS * 10 );

		Enwikuna_Helper_Encryptor::get_instance()->auto_insert_key();
		Enwikuna_Helper_Uninstall::remove_admin_notices();
		Enwikuna_Helper_Uninstall::clear_scheduled_hooks();

		$this->create_update_db_tables();
		$this->verify_db_tables();
		$this->update_functions();
		$this->create_cron_jobs();
		$this->update_enwikuna_helper_version();

		ewkh_delete_transient( 'installing' );

		/**
		 * This hook fires after the plugin is installed successfully
		 *
		 * @wp-hook ewkh_installed
		 * @since 1.0.0
		 */
		do_action( 'ewkh_installed' );
	}

	/**
	 * Check for plugin update -> install again to verify data
	 */
	public function admin_init_action(): void {
		if ( ! defined( 'IFRAME_REQUEST' ) && version_compare( $this->current_enwikuna_helper_version, Constants::get_constant( 'ENWIKUNA_HELPER_VERSION' ), '<' ) ) {
			$this->install();

			/**
			 * This hook fires if the plugin has been updated
			 *
			 * @wp-hook ewkh_install_plugin_updated
			 * @since 1.0.0
			 */
			do_action( 'ewkh_install_plugin_updated' );
		}
	}

	/**
	 * Set up the database tables which the plugin needs to function
	 * This is called from `install` method and is executed in-sync when Enwikuna Helper is installed or updated. This can also be called optionally from `verify_base_tables`
	 */
	private function create_update_db_tables(): void {
		if ( ! function_exists( 'dbDelta' ) ) {
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		}

		$this->wpdb->hide_errors();

		dbDelta( $this->get_db_schema() );
	}

	/**
	 * Checks if all database tables are available
	 *
	 * @param bool $execute_db_table_creation
	 *
	 * @return array
	 */
	public function verify_db_tables( bool $execute_db_table_creation = false ): array {
		$missing_db_tables = array();

		if ( ! function_exists( 'dbDelta' ) ) {
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		}

		if ( $execute_db_table_creation ) {
			$this->create_update_db_tables();
		}

		$alterations = dbDelta( $this->get_db_schema(), false );

		// Check if all tables are available -> if simulation returns a creation result, something is not correct case table should be available
		foreach ( $alterations as $table_name => $table_description ) {
			if ( 0 === strpos( $table_description, "Created table $table_name" ) ) {
				$missing_db_tables[] = $table_name;
			}
		}

		if ( count( $missing_db_tables ) > 0 ) {
			ewkh_update_option( 'db_missing_tables', $missing_db_tables );
		} else {
			ewkh_delete_option( 'db_missing_tables' );
		}

		return $missing_db_tables;
	}

	/**
	 * Runs all update functions that are needed for new version
	 */
	private function update_functions(): void {
		$loop = 10;

		/**
		 * This filter filters if the update functions should be used
		 *
		 * @param bool $enable_update_functions If update functions enabled or not
		 *
		 * @wp-hook ewkh_enable_update_functions
		 * @since 2.0.0
		 */
		if ( ! empty( $this->current_enwikuna_helper_version ) && ! empty( $this->version_updates ) && apply_filters( 'ewkh_enable_update_functions', true ) ) {
			foreach ( $this->version_updates as $version => $update_callbacks ) {
				if ( version_compare( $this->current_enwikuna_helper_version, $version, '<' ) ) {
					foreach ( $update_callbacks as $update_callback ) {
						Enwikuna_Helper_Cron_Jobs::get_instance()->schedule_single(
							time() + $loop,
							'enwikuna_helper_update_callback',
							false,
							array(
								'update_callback' => $update_callback,
							)
						);

						$loop += 10;
					}

					ewkh_set_transient( 'updating', 'yes', MINUTE_IN_SECONDS * 30 );
				}
			}
		}
	}

	/**
	 * Creates all needed Cron jobs for the plugin
	 *
	 * @return void
	 */
	private function create_cron_jobs(): void {
		Enwikuna_Helper_Cron_Jobs::get_instance()->create_cron_jobs();
	}

	/**
	 * Update Enwikuna Helper version to current
	 */
	private function update_enwikuna_helper_version(): void {
		ewkh_delete_option( 'version' );
		ewkh_update_option( 'version', Constants::get_constant( 'ENWIKUNA_HELPER_VERSION' ) );
	}

	/**
	 * Add custom links to plugin description on the plugin screen
	 *
	 * @param array $links
	 * @param string $file
	 *
	 * @return array
	 */
	public function filter_plugin_row_meta( array $links, string $file ): array {
		if ( Constants::get_constant( 'ENWIKUNA_HELPER_PLUGIN_BASENAME' ) !== $file ) {
			return $links;
		}

		/**
		 * This filter filters the URL to the plugin documentation
		 *
		 * @param string $documentation_url The URL to the plugin documentation
		 *
		 * @wp-hook ewkh_documentation_url
		 * @since 2.0.0
		 */
		$documentation_url = apply_filters( 'ewkh_documentation_url', 'https://agency.enwikuna.de/en/docs/enwikuna-helper-en/' );

		/**
		 * This filter filters the support URL
		 *
		 * @param string $support_url The URL to the support
		 *
		 * @wp-hook ewkh_support_url
		 * @since 2.0.0
		 */
		$support_url = apply_filters( 'ewkh_support_url', 'mailto:enwikuna-development-enwikuna-helper-support@gitlab.enwikuna.de?subject=Supportanfrage%20zum%20Enwikuna%20Helper' );

		$row_meta = array(
			'documentation' => '<a href="' . esc_url( $documentation_url ) . '" target="_blank">' . esc_html__( 'Documentation', 'enwikuna-helper' ) . '</a>',
			'support'       => '<a href="' . esc_url( $support_url ) . '" target="_blank">' . esc_html__( 'Support', 'enwikuna-helper' ) . '</a>',
		);

		return wp_parse_args( $row_meta, $links );
	}

	/**
	 * Return a list of Enwikuna Helper tables
	 *
	 * @return array
	 */
	public function get_ewkh_tables(): array {
		$prefix = $this->wpdb->prefix;

		/**
		 * This filter filters the plugin tables
		 *
		 * @param array $tables Includes all plugin tables
		 *
		 * @wp-hook ewkh_install_get_tables
		 * @since 2.0.0
		 */
		return apply_filters(
			'ewkh_install_get_tables',
			array(
				$prefix . self::SUPPORT_TABLE_NAME,
			)
		);
	}

	/**
	 * Get DB table schema
	 *
	 * When adding or removing a table, make sure to update the list of tables in Enwikuna_Helper_Install::get_ewkh_tables()
	 *
	 * @return string
	 */
	private function get_db_schema(): string {
		$collate = '';
		$prefix  = $this->wpdb->prefix;

		if ( $this->wpdb->has_cap( 'collation' ) ) {
			$collate = $this->wpdb->get_charset_collate();
		}

		$support_table_name = $prefix . self::SUPPORT_TABLE_NAME;

		return "
		CREATE TABLE $support_table_name (
  			id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
  			service_user_id BIGINT UNSIGNED NOT NULL,
  			date_enabled timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  		    PRIMARY KEY (id)
		) $collate;
		";
	}
}
