<?php

namespace Enwikuna\Enwikuna_Helper\Data_Stores\Abstracts;

defined( 'ABSPATH' ) || exit;

use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Data_Stores\Abstracts\Interfaces\Enwikuna_Helper_Data_Store_Interface;
use wpdb;

/**
 * Class Enwikuna_Helper_Data_Store_Abstract
 *
 * @package Enwikuna\Enwikuna_Helper\Data_Stores\Abstracts
 */
abstract class Enwikuna_Helper_Data_Store_Abstract extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Data_Store_Interface {

	/**
	 * The table of the data store
	 *
	 * @var string
	 */
	protected string $table;

	/**
	 * The primary key of the table
	 *
	 * @var string
	 */
	protected string $primary_key;

	/**
	 * The class of the model
	 *
	 * @var string
	 */
	protected string $model;

	/**
	 * WordPress database
	 *
	 * @var wpdb
	 */
	private wpdb $wpdb;

	/**
	 * Enwikuna_Helper_Data_Store_Abstract constructor
	 */
	public function __construct() {
		global $wpdb; //DB

		$this->wpdb  = $wpdb;
		$this->table = $wpdb->prefix . $this->table;
	}

	/**
	 * Insert into database
	 *
	 * @param array $data
	 *
	 * @return object|false
	 */
	public function insert( array $data ) {
		if ( empty( $data ) || count( $data ) <= 0 || ! $this->table_exists() ) {
			return false;
		}

		$result = $this->wpdb->insert( $this->table, $data );

		if ( ! $result ) {
			return false;
		}

		return $this->find( $this->wpdb->insert_id );
	}

	/**
	 * Returns a single row by its id
	 *
	 * @param int $id
	 *
	 * @return object|false
	 */
	public function find( int $id ) {
		if ( empty( $id ) || ! class_exists( $this->model ) || ! $this->table_exists() ) {
			return false;
		}

		$where = array(
			array(
				'column'  => $this->primary_key,
				'compare' => '=',
				'value'   => $id,
			),
		);

		$find_query = "SELECT * FROM $this->table";
		$find_query = $this->parse_where_conditions( $find_query, $where );

		/**
		 * This filter filters the data store find query
		 *
		 * @param string $find_query The find query
		 *
		 * @wp-hook ewkh_data_store_find_query
		 * @since 2.0.0
		 */
		$find_query = trim( apply_filters( 'ewkh_data_store_find_query', $find_query ) );

		$result = $this->wpdb->get_row( $find_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $result ) {
			return false;
		}

		return new $this->model( $result );
	}

	/**
	 * Returns a single row by the provided where params
	 *
	 * @param array $where
	 * @param array|null $order_by
	 *
	 * @return object|false
	 */
	public function find_by( array $where, array $order_by = null ) {
		if ( ( ! empty( $where ) && ! is_array( $where ) ) || ! class_exists( $this->model ) || ! $this->table_exists() ) {
			return false;
		}

		$find_by_query = "SELECT * FROM $this->table";
		$find_by_query = $this->parse_where_conditions( $find_by_query, $where );
		$find_by_query = $this->parse_order_by( $find_by_query, $order_by );

		/**
		 * This filter filters the data store find by query
		 *
		 * @param string $find_by_query The find by query
		 *
		 * @wp-hook ewkh_data_store_find_by_query
		 * @since 2.0.0
		 */
		$find_by_query = trim( apply_filters( 'ewkh_data_store_find_by_query', $find_by_query ) );

		$result = $this->wpdb->get_row( $find_by_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $result ) {
			return false;
		}

		return new $this->model( $result );
	}

	/**
	 * Returns all table rows as an array
	 *
	 * @param array|null $order_by // SORT_DESC = 3, SORT_ASC = 4
	 * @param int|null $limit
	 * @param int|null $offset
	 *
	 * @return array|false
	 */
	public function find_all( array $order_by = null, int $limit = null, int $offset = null ) {
		if ( ! class_exists( $this->model ) || ! $this->table_exists() ) {
			return false;
		}

		$all_findings   = array();
		$find_all_query = "SELECT * FROM $this->table";
		$find_all_query = $this->parse_order_by( $find_all_query, $order_by );
		$find_all_query = $this->parse_limit( $find_all_query, $limit );
		$find_all_query = $this->parse_offset( $find_all_query, $offset );

		/**
		 * This filter filters the data store find all query
		 *
		 * @param string $find_all_query The find all query
		 *
		 * @wp-hook ewkh_data_store_find_all_query
		 * @since 2.0.0
		 */
		$find_all_query = trim( apply_filters( 'ewkh_data_store_find_all_query', $find_all_query ) );

		$results = $this->wpdb->get_results( $find_all_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $results ) {
			return false;
		}

		foreach ( $results as $row ) {
			$all_findings[] = new $this->model( $row );
		}

		return $all_findings;
	}

	/**
	 * Retrieves multiple table rows as an array, filtered by the where
	 *
	 * @param array $where
	 * @param array|null $order_by
	 * @param int|null $limit
	 * @param int|null $offset
	 *
	 * @return array|false
	 */
	public function find_all_by( array $where, array $order_by = null, int $limit = null, int $offset = null ) {
		if ( ( ! empty( $where ) && ! is_array( $where ) ) || ! class_exists( $this->model ) || ! $this->table_exists() ) {
			return false;
		}

		$all_findings      = array();
		$find_all_by_query = "SELECT * FROM $this->table";
		$find_all_by_query = $this->parse_where_conditions( $find_all_by_query, $where );
		$find_all_by_query = $this->parse_order_by( $find_all_by_query, $order_by );
		$find_all_by_query = $this->parse_limit( $find_all_by_query, $limit );
		$find_all_by_query = $this->parse_offset( $find_all_by_query, $offset );

		/**
		 * This filter filters the data store find all by query
		 *
		 * @param string $find_all_by_query The find all by query
		 *
		 * @wp-hook ewkh_data_store_find_all_by_query
		 * @since 2.0.0
		 */
		$find_all_by_query = trim( apply_filters( 'ewkh_data_store_find_all_by_query', $find_all_by_query ) );

		$results = $this->wpdb->get_results( $find_all_by_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $results ) {
			return false;
		}

		foreach ( $results as $row ) {
			$all_findings[] = new $this->model( $row );
		}

		return $all_findings;
	}

	/**
	 * Updates a single table row by its ID
	 *
	 * @param array $data
	 * @param int $id
	 * @param bool $return_updated
	 *
	 * @return bool|object
	 */
	public function update( array $data, int $id, bool $return_updated = false ) {
		if ( empty( $data ) || empty( $id ) || ! is_array( $data ) || count( $data ) <= 0 || ! $this->table_exists() ) {
			return false;
		}

		$result = $this->wpdb->update( $this->table, $data, array( $this->primary_key => $id ) );

		if ( ! $result && ! empty( $this->wpdb->last_error ) ) {
			return false;
		}

		if ( $return_updated ) {
			return $this->find( $id );
		}

		return true;
	}

	/**
	 * Updates one or multiple table rows by given where
	 *
	 * @param array $data
	 * @param array $where
	 * @param bool $return_updated
	 *
	 * @return bool|array
	 */
	public function update_by( array $data, array $where, bool $return_updated = false ) {
		if ( empty( $data ) || empty( $where ) || ! is_array( $data ) || ! is_array( $where ) || count( $data ) <= 0 || count( $where ) <= 0 || ! $this->table_exists() ) {
			return false;
		}

		$result = $this->wpdb->update( $this->table, $data, $where );

		if ( ! $result ) {
			return false;
		}

		if ( $return_updated ) {
			return $this->find_all_by( $this->reformat_to_extended_where( $where ) );
		}

		return true;
	}

	/**
	 * Removes multiple table rows by their ID's.
	 *
	 * @param array $ids
	 *
	 * @return bool
	 */
	public function delete( array $ids ): bool {
		if ( empty( $ids ) || ! $this->table_exists() ) {
			return false;
		}

		$ids_str      = implode( ', ', array_map( 'absint', $ids ) );
		$delete_query = "DELETE FROM $this->table WHERE $this->primary_key IN ($ids_str);";

		/**
		 * This filter filters the data store delete query
		 *
		 * @param string $delete_query The delete query
		 *
		 * @wp-hook ewkh_data_store_delete_query
		 * @since 2.0.0
		 */
		$delete_query = apply_filters( 'ewkh_data_store_delete_query', $delete_query );

		$result = $this->wpdb->query( $delete_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $result ) {
			return false;
		}

		return true;
	}

	/**
	 * Delete from database
	 *
	 * @param array $where
	 *
	 * @return bool
	 */
	public function delete_by( array $where ): bool {
		if ( empty( $where ) || ! $this->table_exists() ) {
			return false;
		}

		$result = $this->wpdb->delete( $this->table, $where );

		if ( ! $result ) {
			return false;
		}

		return true;
	}

	/**
	 * Retrieves the total count of table entries
	 *
	 * @return int|false
	 */
	public function count() {
		if ( ! $this->table_exists() ) {
			return false;
		}

		$result = $this->wpdb->get_var( "SELECT COUNT(*) FROM $this->table" ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared

		if ( ! $result ) {
			return false;
		}

		return (int) $result;
	}

	/**
	 * Retrieves the total count of table entries, filtered by the query parameter
	 *
	 * @param array $where
	 *
	 * @return int|false
	 */
	public function count_by( array $where ) {
		if ( empty( $where ) || ! is_array( $where ) || count( $where ) <= 0 || ! $this->table_exists() ) {
			return false;
		}

		$count_by_query = "SELECT COUNT(*) FROM $this->table";
		$count_by_query = $this->parse_where_conditions( $count_by_query, $where );

		/**
		 * This filter filters the data store count by query
		 *
		 * @param string $count_by_query The count by query
		 *
		 * @wp-hook ewkh_data_store_count_by_query
		 * @since 2.0.0
		 */
		$count_by_query = apply_filters( 'ewkh_data_store_count_by_query', $count_by_query );

		$result = $this->wpdb->get_var( trim( $count_by_query ) ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared

		if ( ! $result ) {
			return false;
		}

		return (int) $result;
	}

	/**
	 * Checks if a data row exists
	 *
	 * @param array $where
	 *
	 * @return bool
	 */
	public function exists_by( array $where ): bool {
		if ( empty( $where ) || ! is_array( $where ) || count( $where ) <= 0 || ! $this->table_exists() ) {
			return false;
		}

		$exists_by_query  = "SELECT IF(EXISTS (SELECT id FROM $this->table";
		$exists_by_query  = $this->parse_where_conditions( $exists_by_query, $where );
		$exists_by_query .= '), true, false)';

		/**
		 * This filter filters the data store exists by query
		 *
		 * @param string $exists_by_query The exists by query
		 *
		 * @wp-hook ewkh_data_store_exists_by_query
		 * @since 2.0.0
		 */
		$exists_by_query = apply_filters( 'ewkh_data_store_exists_by_query', $exists_by_query );

		return $this->wpdb->get_var( trim( $exists_by_query ) ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Reformat where to extended where
	 *
	 * @param array $where
	 *
	 * @return array
	 */
	public function reformat_to_extended_where( array $where ): array {
		$extended_where = array();

		foreach ( $where as $key => $value ) {
			$extended_where[] = array(
				'column'  => $key,
				'compare' => '=',
				'value'   => $value,
			);
		}

		return $extended_where;
	}

	/**
	 * Performs a general query on the table
	 *
	 * @param string $query
	 * @param bool $single
	 *
	 * @return array|object|string|null
	 */
	public function query( string $query, bool $single = true ) {
		if ( $single ) {
			return $this->wpdb->get_var( $query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		}

		return $this->wpdb->get_results( $query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Truncates the table
	 *
	 * @return bool|int
	 */
	public function truncate() {
		if ( ! $this->table_exists() ) {
			return false;
		}

		return $this->wpdb->query( "TRUNCATE TABLE $this->table" ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
	}

	/**
	 * Returns the columns for a table
	 *
	 * @return array
	 */
	public function get_columns(): array {
		return $this->wpdb->get_col( "DESC $this->table" ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
	}

	/**
	 * Returns the table
	 *
	 * @return string
	 */
	public function get_table(): string {
		return $this->table;
	}

	/**
	 * Returns the primary key
	 *
	 * @return string
	 */
	public function get_primary_key(): string {
		return $this->primary_key;
	}

	/**
	 * Returns the model
	 *
	 * @return string
	 */
	public function get_model(): string {
		return $this->model;
	}

	/**
	 * Parses the where conditions to be working inside an SQL query
	 *
	 * @param string $query
	 * @param array $where
	 *
	 * @return string
	 */
	private function parse_where_conditions( string $query, array $where ): string {
		$parsed_where = ' WHERE 1=1 ';

		if ( empty( $where ) || ! is_array( $where ) || count( $where ) <= 0 ) {
			return $query;
		}

		$values = array();

		foreach ( $where as $where_details ) {
			$column          = $where_details['column'];
			$is_inner_select = false;

			if ( ! empty( $where_details['table'] ) && ! empty( $where_details['foreign_column'] ) && ! empty( $where_details['primary_column'] ) ) {
				$table = $this->wpdb->prefix . $where_details['table'];

				if ( ! $this->table_exists( $table ) ) {
					continue; // IF table does not exists, do not process where detail
				}

				$is_inner_select = true;
				$foreign_column  = $where_details['foreign_column'];
				$primary_column  = $where_details['primary_column'];
				$parsed_where   .= "AND $foreign_column IN (SELECT $primary_column FROM $table WHERE 1=1 ";
			}

			if ( ! empty( $where_details['group'] ) ) {
				$operator      = $where_details['group']['operator'];
				$parsed_where .= 'AND (';

				foreach ( $where_details['group'] as $group_entry_key => $group_entry ) {
					if ( 'operator' === $group_entry_key ) {
						continue;
					}

					$compare = $group_entry['compare'];

					if ( isset( $group_entry['value'] ) ) {
						$value         = $group_entry['value'];
						$placeholder   = $this->get_where_condition_placeholder( $value );
						$parsed_where .= "$column $compare $placeholder";
						$values[]      = $value;
					} else {
						$parsed_where .= "$column $compare";
					}

					if ( array_key_last( $where_details['group'] ) !== $group_entry_key ) {
						$parsed_where .= " $operator ";
					}
				}

				$parsed_where .= ') ';
			} else {
				$compare = $where_details['compare'] ?? '';

				if ( isset( $where_details['value'] ) ) {
					$value = $where_details['value'];

					if ( is_array( $value ) ) {
						$parsed_where      .= "AND $column IN (";
						$last_sub_value_key = array_key_last( $value );

						foreach ( $value as $sub_value_key => $sub_value ) {
							$parsed_where .= $this->get_where_condition_placeholder( $sub_value );

							if ( $sub_value_key !== $last_sub_value_key ) {
								$parsed_where .= ', ';
							}

							$values[] = $sub_value;
						}

						$parsed_where .= ')';
					} else {
						$placeholder   = $this->get_where_condition_placeholder( $value );
						$parsed_where .= "AND $column $compare $placeholder ";
						$values[]      = $value;
					}
				} else {
					$parsed_where .= "AND $column $compare ";
				}
			}

			if ( $is_inner_select ) {
				$parsed_where .= ')';
			}
		}

		$query .= rtrim( $parsed_where );

		if ( empty( $values ) ) {
			return $query;
		}

		return $this->wpdb->prepare( $query, $values ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Returns the correct placeholder wpdb::prepare and the where clause
	 *
	 * @param mixed $value
	 *
	 * @return string
	 */
	private function get_where_condition_placeholder( $value ): string {
		if ( is_numeric( $value ) ) {
			return '%d';
		}

		if ( is_float( $value ) ) {
			return '%f';
		}

		return '%s';
	}

	/**
	 * Parses the order by conditions to be working inside an SQL query
	 *
	 * @param string $query
	 * @param array|null $order_by
	 *
	 * @return string
	 */
	private function parse_order_by( string $query, ?array $order_by ): string {
		if ( empty( $order_by ) || ! is_array( $order_by ) || count( $order_by ) <= 0 ) {
			return $query;
		}

		$column_name = key( $order_by );
		$sort        = $order_by[ $column_name ];

		if ( ! in_array( $sort, array( SORT_DESC, SORT_ASC ), true ) ) {
			$sort = 4;
		}

		switch ( $sort ) {
			case SORT_DESC:
				$sort = 'DESC';
				break;
			case SORT_ASC:
				$sort = 'ASC';
				break;
		}

		/**
		 * This filter filters the data store exists by query
		 *
		 * @param string $additional_directives The additional order by directives
		 *
		 * @wp-hook ewkh_data_store_additional_order_by_directives
		 * @since 2.0.0
		 */
		$additional_directives = apply_filters( 'ewkh_data_store_additional_order_by_directives', '' );

		return $query . " ORDER BY $column_name $additional_directives $sort";
	}

	/**
	 * Parse the given limit to be working inside an SQL query
	 *
	 * @param string $query
	 * @param int|null $limit
	 *
	 * @return string
	 */
	private function parse_limit( string $query, ?int $limit ): string {
		if ( null === $limit ) {
			return $query;
		}

		return $this->wpdb->prepare( $query . ' LIMIT %d', $limit ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Parse the given offset to be working inside an SQL query
	 *
	 * @param string $query
	 * @param int|null $offset
	 *
	 * @return string
	 */
	private function parse_offset( string $query, ?int $offset ): string {
		if ( null === $offset ) {
			return $query;
		}

		return $this->wpdb->prepare( $query . ' OFFSET %d', $offset ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
	}

	/**
	 * Checks if the table which will be used exists
	 *
	 * @param string|null $table
	 *
	 * @return bool
	 */
	private function table_exists( string $table = null ): bool {
		if ( ! empty( $table ) ) {
			return $this->wpdb->get_var( "SHOW TABLES LIKE '$table'" ) === $table; // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		}

		return $this->wpdb->get_var( "SHOW TABLES LIKE '$this->table'" ) === $this->table; // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
	}
}
