<?php

namespace Enwikuna\Enwikuna_Helper\Data_Stores\Abstracts\Interfaces;

defined( 'ABSPATH' ) || exit;

/**
 * Interface Enwikuna_Helper_Data_Store_Interface
 *
 * @package Enwikuna\Enwikuna_Helper\Data_Stores\Abstracts\Interfaces
 */
interface Enwikuna_Helper_Data_Store_Interface {

	/**
	 * Enwikuna_Helper_Data_Store_Abstract constructor
	 */
	public function __construct();

	/**
	 * Insert into database
	 *
	 * @param array $data
	 *
	 * @return object|false
	 */
	public function insert( array $data );

	/**
	 * Returns a single row by its id
	 *
	 * @param int $id
	 *
	 * @return object|false
	 */
	public function find( int $id );

	/**
	 * Returns a single row by the provided where params
	 *
	 * @param array $where
	 * @param array|null $order_by
	 *
	 * @return object|false
	 */
	public function find_by( array $where, array $order_by = null );

	/**
	 * Returns all table rows as an array
	 *
	 * @param array|null $order_by // SORT_DESC = 3, SORT_ASC = 4
	 * @param int|null $limit
	 * @param int|null $offset
	 *
	 * @return array|false
	 */
	public function find_all( array $order_by = null, int $limit = null, int $offset = null );

	/**
	 * Retrieves multiple table rows as an array, filtered by the where
	 *
	 * @param array $where
	 * @param array|null $order_by
	 * @param int|null $limit
	 * @param int|null $offset
	 *
	 * @return array|false
	 */
	public function find_all_by( array $where, array $order_by = null, int $limit = null, int $offset = null );

	/**
	 * Updates a single table row by its ID
	 *
	 * @param array $data
	 * @param int $id
	 * @param bool $return_updated
	 *
	 * @return bool|object
	 */
	public function update( array $data, int $id, bool $return_updated = false );

	/**
	 * Updates one or multiple table rows by given where
	 *
	 * @param array $data
	 * @param array $where
	 * @param bool $return_updated
	 *
	 * @return bool|array
	 */
	public function update_by( array $data, array $where, bool $return_updated = false );

	/**
	 * Removes multiple table rows by their ID's.
	 *
	 * @param array $ids
	 *
	 * @return bool
	 */
	public function delete( array $ids ): bool;

	/**
	 * Delete from database
	 *
	 * @param array $where
	 *
	 * @return bool
	 */
	public function delete_by( array $where ): bool;

	/**
	 * Retrieves the total count of table entries
	 *
	 * @return int|false
	 */
	public function count();

	/**
	 * Retrieves the total count of table entries, filtered by the query parameter
	 *
	 * @param array $where
	 *
	 * @return int|false
	 */
	public function count_by( array $where );

	/**
	 * Checks if a data row exists
	 *
	 * @param array $where
	 *
	 * @return bool
	 */
	public function exists_by( array $where ): bool;

	/**
	 * Reformat where to extended where
	 *
	 * @param array $where
	 *
	 * @return array
	 */
	public function reformat_to_extended_where( array $where ): array;

	/**
	 * Performs a general query on the table
	 *
	 * @param string $query
	 * @param bool $single
	 *
	 * @return array|object|string|null
	 */
	public function query( string $query, bool $single = true );

	/**
	 * Truncates the table
	 *
	 * @return bool|int
	 */
	public function truncate();

	/**
	 * Returns the columns for a table
	 *
	 * @return array
	 */
	public function get_columns(): array;

	/**
	 * Returns the table
	 *
	 * @return string
	 */
	public function get_table(): string;

	/**
	 * Returns the primary key
	 *
	 * @return string
	 */
	public function get_primary_key(): string;

	/**
	 * Returns the model
	 *
	 * @return string
	 */
	public function get_model(): string;
}
