<?php

namespace Enwikuna\Enwikuna_Helper\Gateways;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Product_Abstract;
use Enwikuna\Enwikuna_Helper\Abstracts\Enwikuna_Helper_Singleton_Abstract;
use Enwikuna\Enwikuna_Helper\Enwikuna_Helper_Encryptor;
use Enwikuna\Enwikuna_Helper\Gateways\HTTP\Enwikuna_Helper_Request;
use Enwikuna\Enwikuna_Helper\Gateways\Interfaces\Enwikuna_Helper_Enwikuna_Agency_Gateway_Interface;

/**
 * Class Enwikuna_Helper_Enwikuna_Agency_Gateway
 *
 * @package Enwikuna\Enwikuna_Helper\Gateways
 */
class Enwikuna_Helper_Enwikuna_Agency_Gateway extends Enwikuna_Helper_Singleton_Abstract implements Enwikuna_Helper_Enwikuna_Agency_Gateway_Interface {

	/**
	 * Enwikuna Agency request
	 *
	 * @var Enwikuna_Helper_Request
	 */
	private Enwikuna_Helper_Request $enwikuna_helper_request;

	/**
	 * Default license key of Enwikuna Helper
	 *
	 * @var string
	 */
	private string $enwikuna_helper_license_key = 'AAAA-AAAA-AAAA-AAAA';

	/**
	 * Enwikuna_Helper_Enwikuna_Agency_Gateway constructor
	 */
	public function __construct() {
		$this->enwikuna_helper_request = new Enwikuna_Helper_Request();
	}

	/**
	 * Checks if the connection to Enwikuna Agency is successful and stable
	 *
	 * @return bool
	 */
	public function is_connected(): bool {
		return $this->enwikuna_helper_request->request( 'ping' )->get_response( 'ping' );
	}

	/**
	 * Activates a product license
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 * @param string $license_key
	 *
	 * @return bool
	 */
	public function activate_license( Enwikuna_Helper_Product_Abstract $product, string $license_key ): bool {
		$args = array(
			'method' => 'POST',
			'body'   => array(
				'host'         => home_url(),
				'product_uuid' => $product->get_id(),
			),
		);

		$response = $this->enwikuna_helper_request->request( 'licenses/activate', $license_key, $args )->get_response();

		if ( ! empty( $response ) && ! is_wp_error( $response ) ) {
			$expires_at = ! empty( $response->data->expires_at ) ? $response->data->expires_at : '';
			$status     = ! empty( $response->data->status ) ? $response->data->status : '';

			$product->register( $license_key, $expires_at, $status );

			return true;
		}

		return false;
	}

	/**
	 * Deactivates a product license
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 *
	 * @return bool
	 */
	public function deactivate_license( Enwikuna_Helper_Product_Abstract $product ): bool {
		$args = array(
			'method' => 'POST',
			'body'   => array(
				'host' => home_url(),
			),
		);

		$response = $this->enwikuna_helper_request->request( 'licenses/deactivate', $product->get_decrypted_license_key(), $args )->get_response();

		if ( ! empty( $response ) && ! is_wp_error( $response ) ) {
			$product->unregister();

			return true;
		}

		return false;
	}

	/**
	 * Returns info about the license
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 *
	 * @return false|object
	 */
	public function license_info( Enwikuna_Helper_Product_Abstract $product ) {
		if ( ! $product->is_registered() ) {
			return false;
		}

		$license_info_cache = ewkh_get_transient( $this->get_license_info_cache_key( $product ) );

		if ( ! empty( $license_info_cache ) ) {
			if ( isset( $license_info_cache->license_key ) ) {
				$license_info_cache->license_key = Enwikuna_Helper_Encryptor::get_instance()->maybe_decrypt( $license_info_cache->license_key );
			}

			return $license_info_cache;
		}

		$response = $this->enwikuna_helper_request->request( 'licenses', $product->get_decrypted_license_key() )->get_response();

		if ( ! empty( $response ) && ! is_wp_error( $response ) ) {
			$cache_data = $response->data;

			if ( isset( $cache_data->license_key ) ) {
				$cache_data->license_key = Enwikuna_Helper_Encryptor::get_instance()->maybe_encrypt( $cache_data->license_key );
			}

			ewkh_set_transient( $this->get_license_info_cache_key( $product ), $cache_data, 6 * Constants::get_constant( 'HOUR_IN_SECONDS' ) );

			return $response->data;
		}

		return false;
	}

	/**
	 * Returns the latest update including the version for a product
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 *
	 * @return false|object
	 */
	public function update_info( Enwikuna_Helper_Product_Abstract $product ) {
		if ( ! $product->is_registered() ) {
			return false;
		}

		$update_info_cache = ewkh_get_transient( $this->get_update_info_cache_key( $product ) );

		if ( ! empty( $update_info_cache ) && ! isset( $_GET['force-check'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if ( isset( $update_info_cache->license_key ) ) {
				$update_info_cache->license_key = Enwikuna_Helper_Encryptor::get_instance()->maybe_decrypt( $update_info_cache->license_key );
			}

			return $update_info_cache;
		}

		if ( 'enwikuna-helper' === $product->get_slug() ) {
			$license_key = $this->enwikuna_helper_license_key;
		} else {
			$license_key = $product->get_decrypted_license_key();
		}

		$response = $this->enwikuna_helper_request->request( 'products/update', $license_key )->get_response();

		if ( ! empty( $response ) && ! is_wp_error( $response ) ) {
			$cache_data = $response->data;

			if ( isset( $cache_data->license_key ) ) {
				$cache_data->license_key = Enwikuna_Helper_Encryptor::get_instance()->maybe_encrypt( $cache_data->license_key );
			}

			ewkh_set_transient( $this->get_update_info_cache_key( $product ), $cache_data, 6 * Constants::get_constant( 'HOUR_IN_SECONDS' ) );

			return $response->data;
		}

		return false;
	}

	/**
	 * Returns update info cache key
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 *
	 * @return string
	 */
	private function get_update_info_cache_key( Enwikuna_Helper_Product_Abstract $product ): string {
		return "update_info_cache_{$product->get_id()}";
	}

	/**
	 * Returns license info cache key
	 *
	 * @param Enwikuna_Helper_Product_Abstract $product
	 *
	 * @return string
	 */
	private function get_license_info_cache_key( Enwikuna_Helper_Product_Abstract $product ): string {
		return "license_info_cache_{$product->get_id()}";
	}
}
